@@ -94,27 +94,11 @@ set_contents (GtkWidget *area,
 }
 
 static void
-info_bar_add_button_with_text (GtkInfoBar  *info_bar,
-			       const gchar *text,
-			       const gchar *icon_name,
-			       gint         response_id)
-{
-	GtkWidget *button;
-	GtkWidget *image;
-
-	button = gtk_info_bar_add_button (info_bar, text, response_id);
-	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image (GTK_BUTTON (button), image);
-}
-
-static void
-set_info_bar_text_and_icon (GtkWidget   *info_bar,
-			    const gchar *icon_name,
-			    const gchar *primary_text,
-			    const gchar *secondary_text)
+set_info_bar_text (GtkWidget   *info_bar,
+		   const gchar *primary_text,
+		   const gchar *secondary_text)
 {
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	gchar *primary_markup;
 	gchar *secondary_markup;
@@ -123,10 +107,6 @@ set_info_bar_text_and_icon (GtkWidget   *info_bar,
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_valign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
@@ -165,23 +145,21 @@ create_io_loading_error_info_bar (const gchar *primary_text,
 {
 	GtkWidget *info_bar;
 
-	info_bar = gtk_info_bar_new_with_buttons (
-					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					NULL);
+	info_bar = gtk_info_bar_new_with_buttons (_("_Cancel"),
+						  GTK_RESPONSE_CANCEL,
+						  NULL);
 	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar),
 				       GTK_MESSAGE_ERROR);
 
-	set_info_bar_text_and_icon (info_bar,
-				    "dialog-error",
-				    primary_text,
-				    secondary_text);
+	set_info_bar_text (info_bar,
+			   primary_text,
+			   secondary_text);
 
 	if (recoverable_error)
 	{
-		info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-					       _("_Retry"),
-					       "view-refresh",
-					       GTK_RESPONSE_OK);
+		gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+					 _("_Retry"),
+					 GTK_RESPONSE_OK);
 	}
 
 	return info_bar;
@@ -200,7 +178,7 @@ parse_gio_error (gint          code,
 	{
 		case G_IO_ERROR_NOT_FOUND:
 		case G_IO_ERROR_NOT_DIRECTORY:
-			*error_message = g_strdup_printf (_("Could not find the file %s."),
+			*error_message = g_strdup_printf (_("Could not find the file “%s”."),
 							  uri_for_display);
 			*message_details = g_strdup (_("Please check that you typed the "
 					      	       "location correctly and try again."));
@@ -218,16 +196,16 @@ parse_gio_error (gint          code,
 
 				if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
 				{
-					scheme_markup = g_markup_printf_escaped ("<i>%s:</i>", scheme_string);
+					scheme_markup = g_markup_escape_text (scheme_string, -1);
 
 					/* Translators: %s is a URI scheme (like for example http:, ftp:, etc.) */
-					*message_details = g_strdup_printf (_("gedit cannot handle %s locations."),
+					*message_details = g_strdup_printf (_("Unable to handle “%s:” locations."),
 									   scheme_markup);
 					g_free (scheme_markup);
 				}
 				else
 				{
-					*message_details = g_strdup (_("gedit cannot handle this location."));
+					*message_details = g_strdup (_("Unable to handle this location."));
 				}
 
 				g_free (scheme_string);
@@ -235,22 +213,19 @@ parse_gio_error (gint          code,
 			break;
 
 		case G_IO_ERROR_NOT_MOUNTABLE_FILE:
-			*message_details = g_strdup (_("The location of the file cannot be mounted."));
-			break;
-
 		case G_IO_ERROR_NOT_MOUNTED:
-			*message_details = g_strdup( _("The location of the file cannot be accessed because it is not mounted."));
-
+			*message_details = g_strdup (_("The location of the file cannot be accessed."));
 			break;
+
 		case G_IO_ERROR_IS_DIRECTORY:
-			*error_message = g_strdup_printf (_("%s is a directory."),
+			*error_message = g_strdup_printf (_("“%s” is a directory."),
 							 uri_for_display);
 			*message_details = g_strdup (_("Please check that you typed the "
 						      "location correctly and try again."));
 			break;
 
 		case G_IO_ERROR_INVALID_FILENAME:
-			*error_message = g_strdup_printf (_("%s is not a valid location."),
+			*error_message = g_strdup_printf (_("“%s” is not a valid location."),
 							 uri_for_display);
 			*message_details = g_strdup (_("Please check that you typed the "
 						      "location correctly and try again."));
@@ -281,12 +256,12 @@ parse_gio_error (gint          code,
 						host_name = gedit_utils_make_valid_utf8 (hn);
 						g_free (hn);
 
-						host_markup = g_markup_printf_escaped ("<i>%s</i>", host_name);
+						host_markup = g_markup_escape_text (host_name, -1);
 						g_free (host_name);
 
 						/* Translators: %s is a host name */
 						*message_details = g_strdup_printf (
-							_("Host %s could not be found. "
+							_("Host “%s” could not be found. "
 							"Please check that your proxy settings "
 							"are correct and try again."),
 							host_markup);
@@ -309,7 +284,7 @@ parse_gio_error (gint          code,
 			break;
 
 		case G_IO_ERROR_NOT_REGULAR_FILE:
-			*message_details = g_strdup_printf (_("%s is not a regular file."),
+			*message_details = g_strdup_printf (_("“%s” is not a regular file."),
 							   uri_for_display);
 			break;
 
@@ -409,12 +384,12 @@ gedit_unrecoverable_reverting_error_info_bar_new (GFile        *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	if (is_gio_error (error, G_IO_ERROR_NOT_FOUND))
 	{
-		message_details = g_strdup (_("gedit cannot find the file. "
+		message_details = g_strdup (_("Cannot find the requested file. "
 					      "Perhaps it has recently been deleted."));
 	}
 	else
@@ -424,7 +399,7 @@ gedit_unrecoverable_reverting_error_info_bar_new (GFile        *location,
 
 	if (error_message == NULL)
 	{
-		error_message = g_strdup_printf (_("Could not revert the file %s."),
+		error_message = g_strdup_printf (_("Could not revert the file “%s”."),
 						 uri_for_display);
 	}
 
@@ -484,7 +459,6 @@ create_conversion_error_info_bar (const gchar *primary_text,
 {
 	GtkWidget *info_bar;
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	gchar *primary_markup;
 	gchar *secondary_markup;
@@ -493,10 +467,9 @@ create_conversion_error_info_bar (const gchar *primary_text,
 
 	info_bar = gtk_info_bar_new ();
 
-	info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-				       _("_Retry"),
-				       "edit-redo",
-				       GTK_RESPONSE_OK);
+	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+				 _("_Retry"),
+				 GTK_RESPONSE_OK);
 
 	if (edit_anyway)
 	{
@@ -515,16 +488,11 @@ create_conversion_error_info_bar (const gchar *primary_text,
 	}
 
 	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
-				 GTK_STOCK_CANCEL,
+				 _("_Cancel"),
 				 GTK_RESPONSE_CANCEL);
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name ("dialog-error",
-	                                      GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_valign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
@@ -596,7 +564,7 @@ gedit_io_loading_error_info_bar_new (GFile               *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	if (encoding != NULL)
@@ -616,8 +584,7 @@ gedit_io_loading_error_info_bar_new (GFile               *location,
 	         (error->domain == GEDIT_DOCUMENT_ERROR &&
 	         error->code == GEDIT_DOCUMENT_ERROR_ENCODING_AUTO_DETECTION_FAILED))
 	{
-		message_details = g_strconcat (_("gedit has not been able to detect "
-					         "the character encoding."), "\n",
+		message_details = g_strconcat (_("Unable to detect the character encoding."), "\n",
 					       _("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a character encoding from the menu and try again."), NULL);
 		convert_error = TRUE;
@@ -625,7 +592,7 @@ gedit_io_loading_error_info_bar_new (GFile               *location,
 	else if (error->domain == GEDIT_DOCUMENT_ERROR &&
 	         error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
 	{
-		error_message = g_strdup_printf (_("There was a problem opening the file %s."),
+		error_message = g_strdup_printf (_("There was a problem opening the file “%s”."),
 						 uri_for_display);
 		message_details = g_strconcat (_("The file you opened has some invalid characters. "
 					       "If you continue editing this file you could corrupt this "
@@ -637,7 +604,7 @@ gedit_io_loading_error_info_bar_new (GFile               *location,
 	}
 	else if (is_gio_error (error, G_IO_ERROR_INVALID_DATA) && encoding != NULL)
 	{
-		error_message = g_strdup_printf (_("Could not open the file %s using the %s character encoding."),
+		error_message = g_strdup_printf (_("Could not open the file “%s” using the “%s” character encoding."),
 						 uri_for_display,
 						 encoding_name);
 		message_details = g_strconcat (_("Please check that you are not trying to open a binary file."), "\n",
@@ -651,7 +618,7 @@ gedit_io_loading_error_info_bar_new (GFile               *location,
 
 	if (error_message == NULL)
 	{
-		error_message = g_strdup_printf (_("Could not open the file %s."),
+		error_message = g_strdup_printf (_("Could not open the file “%s”."),
 						 uri_for_display);
 	}
 
@@ -705,12 +672,12 @@ gedit_conversion_error_while_saving_info_bar_new (GFile               *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	encoding_name = gedit_encoding_to_string (encoding);
 
-	error_message = g_strdup_printf (_("Could not save the file %s using the %s character encoding."),
+	error_message = g_strdup_printf (_("Could not save the file “%s” using the “%s” character encoding."),
 					 uri_for_display,
 					 encoding_name);
 	message_details = g_strconcat (_("The document contains one or more characters that cannot be encoded "
@@ -751,7 +718,6 @@ gedit_file_already_open_warning_info_bar_new (GFile *location)
 {
 	GtkWidget *info_bar;
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	gchar *primary_markup;
 	gchar *secondary_markup;
@@ -775,7 +741,7 @@ gedit_file_already_open_warning_info_bar_new (GFile *location)
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	info_bar = gtk_info_bar_new ();
@@ -794,15 +760,10 @@ gedit_file_already_open_warning_info_bar_new (GFile *location)
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name ("dialog-warning",
-	                                      GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_halign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
-	primary_text = g_strdup_printf (_("This file (%s) is already open in another gedit window."), uri_for_display);
+	primary_text = g_strdup_printf (_("This file “%s” is already open in another window."), uri_for_display);
 	g_free (uri_for_display);
 
 	primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
@@ -816,8 +777,7 @@ gedit_file_already_open_warning_info_bar_new (GFile *location)
 	gtk_widget_set_can_focus (primary_label, TRUE);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
-	secondary_text = _("gedit opened this instance of the file in a non-editable way. "
-			   "Do you want to edit it anyway?");
+	secondary_text = _("Do you want to edit it anyway?");
 	secondary_markup = g_strdup_printf ("<small>%s</small>",
 					    secondary_text);
 	secondary_label = gtk_label_new (secondary_markup);
@@ -841,7 +801,6 @@ gedit_externally_modified_saving_error_info_bar_new (GFile        *location,
 {
 	GtkWidget *info_bar;
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	gchar *primary_markup;
 	gchar *secondary_markup;
@@ -868,15 +827,14 @@ gedit_externally_modified_saving_error_info_bar_new (GFile        *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	info_bar = gtk_info_bar_new ();
 
-	info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-					     _("S_ave Anyway"),
-					     _("_Save"),
-					     GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+				 _("S_ave Anyway"),
+				 GTK_RESPONSE_YES);
 	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
 				 _("D_on't Save"),
 				 GTK_RESPONSE_CANCEL);
@@ -885,11 +843,6 @@ gedit_externally_modified_saving_error_info_bar_new (GFile        *location,
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name ("dialog-warning",
-	                                      GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_valign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
@@ -897,7 +850,7 @@ gedit_externally_modified_saving_error_info_bar_new (GFile        *location,
 	 * could be interpreted as the changes he made in the document. beside "reading" is
 	 * not accurate (since last load/save)
 	 */
-	primary_text = g_strdup_printf (_("The file %s has been modified since reading it."),
+	primary_text = g_strdup_printf (_("The file “%s” has been modified since reading it."),
 					uri_for_display);
 	g_free (uri_for_display);
 
@@ -936,7 +889,6 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 {
 	GtkWidget *info_bar;
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	gchar *primary_markup;
 	gchar *secondary_markup;
@@ -967,15 +919,14 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	info_bar = gtk_info_bar_new ();
 
-	info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-				       _("S_ave Anyway"),
-				       _("_Save"),
-				       GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+				 _("S_ave Anyway"),
+				 GTK_RESPONSE_YES);
 	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
 				 _("D_on't Save"),
 				 GTK_RESPONSE_CANCEL);
@@ -984,11 +935,6 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name ("dialog-warning",
-	                                      GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_valign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
@@ -1001,12 +947,12 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 	/* FIXME: review this messages */
 	if (create_backup_copy)
 	{
-		primary_text = g_strdup_printf (_("Could not create a backup file while saving %s"),
+		primary_text = g_strdup_printf (_("Could not create a backup file while saving “%s”"),
 						uri_for_display);
 	}
 	else
 	{
-		primary_text = g_strdup_printf (_("Could not create a temporary backup file while saving %s"),
+		primary_text = g_strdup_printf (_("Could not create a temporary backup file while saving “%s”"),
 						uri_for_display);
 	}
 
@@ -1023,7 +969,7 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 	gtk_widget_set_can_focus (primary_label, TRUE);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
-	secondary_text = _("gedit could not back up the old copy of the file before saving the new one. "
+	secondary_text = _("Could not back up the old copy of the file before saving the new one. "
 			   "You can ignore this warning and save the file anyway, but if an error "
 			   "occurs while saving, you could lose the old copy of the file. Save anyway?");
 	secondary_markup = g_strdup_printf ("<small>%s</small>",
@@ -1071,7 +1017,7 @@ gedit_unrecoverable_saving_error_info_bar_new (GFile        *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	if (is_gio_error (error, G_IO_ERROR_NOT_SUPPORTED))
@@ -1080,10 +1026,10 @@ gedit_unrecoverable_saving_error_info_bar_new (GFile        *location,
 
 		if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
 		{
-			scheme_markup = g_markup_printf_escaped ("<i>%s:</i>", scheme_string);
+			scheme_markup = g_markup_escape_text (scheme_string, -1);
 
 			/* Translators: %s is a URI scheme (like for example http:, ftp:, etc.) */
-			message_details = g_strdup_printf (_("gedit cannot handle %s locations in write mode. "
+			message_details = g_strdup_printf (_("Cannot handle “%s:” locations in write mode. "
 							     "Please check that you typed the "
 							     "location correctly and try again."),
 							   scheme_markup);
@@ -1091,7 +1037,7 @@ gedit_unrecoverable_saving_error_info_bar_new (GFile        *location,
 		}
 		else
 		{
-			message_details = g_strdup (_("gedit cannot handle this location in write mode. "
+			message_details = g_strdup (_("Cannot handle this location in write mode. "
 						      "Please check that you typed the "
 						      "location correctly and try again."));
 		}
@@ -1100,7 +1046,7 @@ gedit_unrecoverable_saving_error_info_bar_new (GFile        *location,
 	}
 	else if (is_gio_error (error, G_IO_ERROR_INVALID_FILENAME))
 	{
-		message_details = g_strdup_printf (_("%s is not a valid location. "
+		message_details = g_strdup_printf (_("“%s” is not a valid location. "
 						     "Please check that you typed the "
 						     "location correctly and try again."),
 						   uri_for_display);
@@ -1152,7 +1098,7 @@ gedit_unrecoverable_saving_error_info_bar_new (GFile        *location,
 
 	if (error_message == NULL)
 	{
-		error_message = g_strdup_printf (_("Could not save the file %s."),
+		error_message = g_strdup_printf (_("Could not save the file “%s”."),
 						 uri_for_display);
 	}
 
@@ -1175,7 +1121,6 @@ gedit_externally_modified_info_bar_new (GFile    *location,
 	gchar *uri_for_display;
 	gchar *temp_uri_for_display;
 	gchar *primary_text;
-	const gchar *secondary_text;
 	GtkWidget *info_bar;
 
 	g_return_val_if_fail (G_IS_FILE (location), NULL);
@@ -1190,38 +1135,47 @@ gedit_externally_modified_info_bar_new (GFile    *location,
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	/* FIXME: review this message, it's not clear since for the user the "modification"
 	 * could be interpreted as the changes he made in the document. beside "reading" is
 	 * not accurate (since last load/save)
 	 */
-	primary_text = g_strdup_printf (_("The file %s changed on disk."),
+	primary_text = g_strdup_printf (_("The file “%s” changed on disk."),
 					uri_for_display);
 	g_free (uri_for_display);
 
-	if (document_modified)
-		secondary_text = _("Do you want to drop your changes and reload the file?");
-	else
-		secondary_text = _("Do you want to reload the file?");
-
 	info_bar = gtk_info_bar_new ();
 
-	info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-				       _("_Reload"),
-				       "view-refresh",
-				       GTK_RESPONSE_OK);
+	if (document_modified)
+	{
+		GtkWidget *box;
+		GtkWidget *button;
+		button = gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+						  _("Drop Changes and _Reload"),
+						  GTK_RESPONSE_OK);
+		box = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+		gtk_button_box_set_child_non_homogeneous (GTK_BUTTON_BOX (box),
+							  button,
+							  TRUE);
+	}
+	else
+	{
+		gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+					 _("_Reload"),
+					 GTK_RESPONSE_OK);
+	}
+
 	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
-				 GTK_STOCK_CANCEL,
+				 _("_Cancel"),
 				 GTK_RESPONSE_CANCEL);
 	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar),
 				       GTK_MESSAGE_WARNING);
 
-	set_info_bar_text_and_icon (info_bar,
-					"gtk-dialog-warning",
-					primary_text,
-					secondary_text);
+	set_info_bar_text (info_bar,
+			   primary_text,
+			   NULL);
 
 	g_free (primary_text);
 
@@ -1233,7 +1187,6 @@ gedit_invalid_character_info_bar_new (GFile *location)
 {
 	GtkWidget *info_bar;
 	GtkWidget *hbox_content;
-	GtkWidget *image;
 	GtkWidget *vbox;
 	GtkWidget *primary_label;
 	GtkWidget *secondary_label;
@@ -1257,15 +1210,14 @@ gedit_invalid_character_info_bar_new (GFile *location)
 								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
+	uri_for_display = g_markup_escape_text (temp_uri_for_display, -1);
 	g_free (temp_uri_for_display);
 
 	info_bar = gtk_info_bar_new ();
 
-	info_bar_add_button_with_text (GTK_INFO_BAR (info_bar),
-				       _("S_ave Anyway"),
-				       _("_Save"),
-				       GTK_RESPONSE_YES);
+	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
+				 _("S_ave Anyway"),
+				 GTK_RESPONSE_YES);
 	gtk_info_bar_add_button (GTK_INFO_BAR (info_bar),
 				 _("D_on't Save"),
 				 GTK_RESPONSE_CANCEL);
@@ -1274,15 +1226,10 @@ gedit_invalid_character_info_bar_new (GFile *location)
 
 	hbox_content = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 8);
 
-	image = gtk_image_new_from_icon_name ("dialog-warning",
-	                                      GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox_content), image, FALSE, FALSE, 0);
-	gtk_widget_set_valign (image, GTK_ALIGN_START);
-
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
-	primary_text = g_strdup_printf (_("Some invalid chars have been detected while saving %s"),
+	primary_text = g_strdup_printf (_("Some invalid chars have been detected while saving “%s”"),
 					uri_for_display);
 
 	g_free (uri_for_display);
