@@ -3,6 +3,7 @@
  * This file is part of gedit
  *
  * Copyright (C) 2010 - Ignacio Casal Quinteiro
+ * Copyright (C) 2013 - Sébastien Wilmet
  *
  * gedit is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,10 +26,11 @@
 #endif
 
 #include "gedit-view-frame.h"
-#include "gedit-marshal.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
+#include "libgd/gd.h"
 
+#include <gtksourceview/gtksource.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <stdlib.h>
@@ -37,8 +39,6 @@
 
 #define SEARCH_POPUP_MARGIN 12
 
-#define GEDIT_VIEW_FRAME_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GEDIT_TYPE_VIEW_FRAME, GeditViewFramePrivate))
-
 typedef enum
 {
 	GOTO_LINE,
@@ -56,20 +56,14 @@ struct _GeditViewFramePrivate
 	 */
 	GtkTextMark *start_mark;
 
-	/* used to restore the search state if an
-	 * incremental search is cancelled
+	/* Used to restore the search state if an incremental search is
+	 * cancelled.
 	 */
 	gchar       *old_search_text;
-	GeditSearchFlags old_search_flags;
-
-	/* used to remember the state of the last
-	 * incremental search (the document search
-	 * state may be changed by the search dialog)
-	 */
-	GeditSearchFlags search_flags;
 
 	GtkRevealer *slider;
-	GtkEntry    *search_entry;
+	GdTaggedEntry *search_entry;
+	GdTaggedEntryTag *entry_tag;
 	GtkWidget   *go_up_button;
 	GtkWidget   *go_down_button;
 
@@ -77,9 +71,23 @@ struct _GeditViewFramePrivate
 	glong        view_scroll_event_id;
 	glong        search_entry_focus_out_id;
 	glong        search_entry_changed_id;
+	guint        idle_update_entry_tag_id;
 
 	guint        disable_popdown : 1;
-	guint        wrap_around : 1;
+
+	/* Used to remember the state of the last incremental search (the
+	 * buffer search state may be changed by the search and replace dialog).
+	 */
+	guint case_sensitive_search : 1;
+	guint search_at_word_boundaries : 1;
+	guint search_wrap_around : 1;
+
+	/* Used to restore the search state if an incremental search is
+	 * cancelled.
+	 */
+	guint old_case_sensitive_search : 1;
+	guint old_search_at_word_boundaries : 1;
+	guint old_search_wrap_around : 1;
 };
 
 enum
@@ -95,7 +103,7 @@ typedef enum
 	GEDIT_SEARCH_ENTRY_NOT_FOUND
 } GeditSearchEntryBgColor;
 
-G_DEFINE_TYPE (GeditViewFrame, gedit_view_frame, GTK_TYPE_OVERLAY)
+G_DEFINE_TYPE_WITH_PRIVATE (GeditViewFrame, gedit_view_frame, GTK_TYPE_OVERLAY)
 
 static void
 gedit_view_frame_finalize (GObject *object)
@@ -118,6 +126,14 @@ gedit_view_frame_dispose (GObject *object)
 		frame->priv->typeselect_flush_timeout = 0;
 	}
 
+	if (frame->priv->idle_update_entry_tag_id != 0)
+	{
+		g_source_remove (frame->priv->idle_update_entry_tag_id);
+		frame->priv->idle_update_entry_tag_id = 0;
+	}
+
+	g_clear_object (&frame->priv->entry_tag);
+
 	G_OBJECT_CLASS (gedit_view_frame_parent_class)->dispose (object);
 }
 
@@ -225,109 +241,11 @@ set_entry_background (GeditViewFrame          *frame,
 	}
 }
 
-static gboolean
-run_search (GeditViewFrame   *frame,
-            const gchar      *entry_text,
-            gboolean          search_backward,
-            gboolean          wrap_around,
-            gboolean          typing)
+static void
+finish_search (GeditViewFrame    *frame,
+	       gboolean           found)
 {
-	GtkTextIter    start_iter;
-	GtkTextIter    match_start;
-	GtkTextIter    match_end;
-	gboolean       found = FALSE;
-	GeditDocument *doc;
-
-	g_return_val_if_fail (frame->priv->search_mode == SEARCH, FALSE);
-
-	doc = gedit_view_frame_get_document (frame);
-
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
-	                                  &start_iter,
-	                                  frame->priv->start_mark);
-
-	if (*entry_text != '\0')
-	{
-		if (!search_backward)
-		{
-			if (!typing)
-			{
-				/* forward and _NOT_ typing */
-				gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),
-				                                      &start_iter,
-				                                      &match_end);
-			}
-
-			/* run search */
-			found = gedit_document_search_forward (doc,
-			                                       &start_iter,
-			                                       NULL,
-			                                       &match_start,
-			                                       &match_end);
-		}
-		else if (!typing)
-		{
-			/* backward and not typing */
-			gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),
-			                                      &start_iter,
-			                                      &match_end);
-
-			/* run search */
-			found = gedit_document_search_backward (doc,
-			                                        NULL,
-			                                        &start_iter,
-			                                        &match_start,
-			                                        &match_end);
-		}
-		else
-		{
-			/* backward (while typing) */
-			g_return_val_if_reached (FALSE);
-		}
-
-		if (!found && wrap_around)
-		{
-			if (!search_backward)
-			{
-				found = gedit_document_search_forward (doc,
-				                                       NULL,
-				                                       NULL, /* FIXME: set the end_inter */
-				                                       &match_start,
-				                                       &match_end);
-			}
-			else
-			{
-				found = gedit_document_search_backward (doc,
-				                                        NULL, /* FIXME: set the start_inter */
-				                                        NULL,
-				                                        &match_start,
-				                                        &match_end);
-			}
-		}
-	}
-	else
-	{
-		gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),
-		                                      &start_iter,
-		                                      NULL);
-	}
-
-	if (found)
-	{
-		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc),
-		                              &match_start);
-
-		gtk_text_buffer_move_mark_by_name (GTK_TEXT_BUFFER (doc),
-		                                   "selection_bound", &match_end);
-	}
-	else if (typing)
-	{
-		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),
-		                                  &start_iter,
-		                                  frame->priv->start_mark);
-		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc),
-		                              &start_iter);
-	}
+	const gchar *entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
 
 	if (found || (*entry_text == '\0'))
 	{
@@ -339,16 +257,162 @@ run_search (GeditViewFrame   *frame,
 	{
 		set_entry_background (frame, GEDIT_SEARCH_ENTRY_NOT_FOUND);
 	}
+}
 
-	return found;
+static void
+start_search_finished (GtkSourceBuffer *buffer,
+		       GAsyncResult    *result,
+		       GeditViewFrame  *frame)
+{
+	GtkTextIter match_start;
+	GtkTextIter match_end;
+	gboolean found;
+
+	found = gtk_source_buffer_forward_search_finish (buffer,
+							 result,
+							 &match_start,
+							 &match_end,
+							 NULL);
+
+	if (found)
+	{
+		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer),
+					      &match_start,
+					      &match_end);
+	}
+	else
+	{
+		GtkTextIter start_at;
+
+		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+						  &start_at,
+						  frame->priv->start_mark);
+
+		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer),
+					      &start_at,
+					      &start_at);
+	}
+
+	finish_search (frame, found);
+}
+
+static void
+start_search (GeditViewFrame *frame)
+{
+	GtkTextIter start_at;
+	GtkSourceBuffer *buffer;
+
+	g_return_if_fail (frame->priv->search_mode == SEARCH);
+
+	buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (buffer),
+					  &start_at,
+					  frame->priv->start_mark);
+
+	gtk_source_buffer_forward_search_async (buffer,
+						&start_at,
+						NULL,
+						(GAsyncReadyCallback)start_search_finished,
+						frame);
+}
+
+static void
+forward_search_finished (GtkSourceBuffer *buffer,
+			 GAsyncResult    *result,
+			 GeditViewFrame  *frame)
+{
+	GtkTextIter match_start;
+	GtkTextIter match_end;
+	gboolean found;
+
+	found = gtk_source_buffer_forward_search_finish (buffer,
+							 result,
+							 &match_start,
+							 &match_end,
+							 NULL);
+
+	if (found)
+	{
+		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer),
+					      &match_start,
+					      &match_end);
+	}
+
+	finish_search (frame, found);
+}
+
+static void
+forward_search (GeditViewFrame *frame)
+{
+	GtkTextIter start_at;
+	GtkSourceBuffer *buffer;
+
+	g_return_if_fail (frame->priv->search_mode == SEARCH);
+
+	buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (buffer),
+					      NULL,
+					      &start_at);
+
+	gtk_source_buffer_forward_search_async (buffer,
+						&start_at,
+						NULL,
+						(GAsyncReadyCallback)forward_search_finished,
+						frame);
+}
+
+static void
+backward_search_finished (GtkSourceBuffer *buffer,
+			  GAsyncResult    *result,
+			  GeditViewFrame  *frame)
+{
+	GtkTextIter match_start;
+	GtkTextIter match_end;
+	gboolean found;
+
+	found = gtk_source_buffer_backward_search_finish (buffer,
+							  result,
+							  &match_start,
+							  &match_end,
+							  NULL);
+
+	if (found)
+	{
+		gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer),
+					      &match_start,
+					      &match_end);
+	}
+
+	finish_search (frame, found);
+}
+
+static void
+backward_search (GeditViewFrame *frame)
+{
+	GtkTextIter start_at;
+	GtkSourceBuffer *buffer;
+
+	g_return_if_fail (frame->priv->search_mode == SEARCH);
+
+	buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (buffer),
+					      &start_at,
+					      NULL);
+
+	gtk_source_buffer_backward_search_async (buffer,
+						 &start_at,
+						 NULL,
+						 (GAsyncReadyCallback)backward_search_finished,
+						 frame);
 }
 
 static void
 search_again (GeditViewFrame *frame,
               gboolean        search_backward)
 {
-	const gchar *entry_text;
-
 	g_return_if_fail (frame->priv->search_mode == SEARCH);
 
 	/* renew the flush timeout */
@@ -361,13 +425,14 @@ search_again (GeditViewFrame *frame,
 					       frame);
 	}
 
-	entry_text = gtk_entry_get_text (frame->priv->search_entry);
-
-	run_search (frame,
-	            entry_text,
-	            search_backward,
-	            frame->priv->wrap_around,
-	            FALSE);
+	if (search_backward)
+	{
+		backward_search (frame);
+	}
+	else
+	{
+		forward_search (frame);
+	}
 }
 
 static gboolean
@@ -414,13 +479,18 @@ search_widget_key_press_event (GtkWidget      *widget,
 	{
 		if (frame->priv->search_mode == SEARCH)
 		{
-			GeditDocument *doc;
+			GtkSourceBuffer *buffer;
+			gchar *unescaped_search_text;
 
 			/* restore document search so that Find Next does the right thing */
-			doc = gedit_view_frame_get_document (frame);
-			gedit_document_set_search_text (doc,
-			                                frame->priv->old_search_text,
-			                                frame->priv->old_search_flags);
+			buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+			gtk_source_buffer_set_case_sensitive_search (buffer, frame->priv->old_case_sensitive_search);
+			gtk_source_buffer_set_search_at_word_boundaries (buffer, frame->priv->old_search_at_word_boundaries);
+			gtk_source_buffer_set_search_wrap_around (buffer, frame->priv->old_search_wrap_around);
+
+			unescaped_search_text = gtk_source_utils_unescape_search_text (frame->priv->old_search_text);
+			gtk_source_buffer_set_search_text (buffer, unescaped_search_text);
+			g_free (unescaped_search_text);
 		}
 
 		hide_search_widget (frame, TRUE);
@@ -466,59 +536,137 @@ search_widget_key_press_event (GtkWidget      *widget,
 }
 
 static void
-update_search (GeditViewFrame *frame)
+update_entry_tag (GeditViewFrame *frame)
 {
-	GeditDocument *doc;
-	const gchar *entry_text;
-	gchar *search_text;
-	GeditSearchFlags search_flags;
+	GtkTextBuffer *buffer;
+	GtkTextIter select_start;
+	GtkTextIter select_end;
+	gint count;
+	gint pos;
+	gchar *label;
 
-	if (frame->priv->search_mode != SEARCH)
+	if (frame->priv->search_mode == GOTO_LINE)
 	{
+		gd_tagged_entry_remove_tag (frame->priv->search_entry,
+					    frame->priv->entry_tag);
 		return;
 	}
 
-	doc = gedit_view_frame_get_document (frame);
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->priv->view));
 
-	entry_text = gtk_entry_get_text (frame->priv->search_entry);
+	count = gtk_source_buffer_get_search_occurrences_count (GTK_SOURCE_BUFFER (buffer));
 
-	search_text = gedit_document_get_search_text (doc, &search_flags);
+	gtk_text_buffer_get_selection_bounds (buffer, &select_start, &select_end);
 
-	if ((search_text == NULL) ||
-	    (strcmp (search_text, entry_text) != 0) ||
-	     search_flags != frame->priv->search_flags)
+	pos = gtk_source_buffer_get_search_occurrence_position (GTK_SOURCE_BUFFER (buffer),
+								&select_start,
+								&select_end);
+
+	if (count == -1 || pos == -1)
 	{
-		gedit_document_set_search_text (doc,
-						entry_text,
-						frame->priv->search_flags);
+		/* Wait that the buffer is fully scanned. */
+		return;
 	}
 
-	g_free (search_text);
+	if (count == 0)
+	{
+		gd_tagged_entry_remove_tag (frame->priv->search_entry,
+					    frame->priv->entry_tag);
+		return;
+	}
+
+	if (pos == 0)
+	{
+		label = g_strdup_printf ("%d", count);
+	}
+	else
+	{
+		/* Translators: the first %d is the position of the current search
+		 * occurrence, and the second %d is the total number of search
+		 * occurrences.
+		 */
+		label = g_strdup_printf (_("%d of %d"), pos, count);
+	}
+
+	gd_tagged_entry_tag_set_label (frame->priv->entry_tag, label);
+
+	gd_tagged_entry_add_tag (frame->priv->search_entry,
+				 frame->priv->entry_tag);
+
+	g_free (label);
+}
+
+static gboolean
+update_entry_tag_idle_cb (GeditViewFrame *frame)
+{
+	frame->priv->idle_update_entry_tag_id = 0;
+
+	update_entry_tag (frame);
+
+	return G_SOURCE_REMOVE;
 }
 
 static void
-wrap_around_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
-                               GeditViewFrame   *frame)
+install_update_entry_tag_idle (GeditViewFrame *frame)
 {
-	frame->priv->wrap_around = gtk_check_menu_item_get_active (checkmenuitem);
+	if (frame->priv->idle_update_entry_tag_id == 0)
+	{
+		frame->priv->idle_update_entry_tag_id = g_idle_add ((GSourceFunc)update_entry_tag_idle_cb,
+								    frame);
+	}
 }
 
 static void
-match_entire_word_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
-                                     GeditViewFrame   *frame)
+update_search (GeditViewFrame *frame)
 {
-	GEDIT_SEARCH_SET_ENTIRE_WORD (frame->priv->search_flags,
-	                              gtk_check_menu_item_get_active (checkmenuitem));
-	update_search (frame);
+	GtkSourceBuffer *buffer;
+	const gchar *entry_text;
+	gchar *unescaped_entry_text;
+
+	buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
+	unescaped_entry_text = gtk_source_utils_unescape_search_text (entry_text);
+
+	gtk_source_buffer_set_search_text (buffer, unescaped_entry_text);
+	gtk_source_buffer_set_case_sensitive_search (buffer, frame->priv->case_sensitive_search);
+	gtk_source_buffer_set_search_at_word_boundaries (buffer, frame->priv->search_at_word_boundaries);
+	gtk_source_buffer_set_search_wrap_around (buffer, frame->priv->search_wrap_around);
+
+	g_free (unescaped_entry_text);
 }
 
 static void
-match_case_menu_item_toggled (GtkCheckMenuItem *checkmenuitem,
-                              GeditViewFrame   *frame)
+wrap_around_toggled_cb (GtkCheckMenuItem *menu_item,
+			GeditViewFrame   *frame)
 {
-	GEDIT_SEARCH_SET_CASE_SENSITIVE (frame->priv->search_flags,
-	                                 gtk_check_menu_item_get_active (checkmenuitem));
-	update_search (frame);
+	GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	frame->priv->search_wrap_around = gtk_check_menu_item_get_active (menu_item);
+
+	gtk_source_buffer_set_search_wrap_around (buffer, frame->priv->search_wrap_around);
+}
+
+static void
+entire_word_toggled_cb (GtkCheckMenuItem *menu_item,
+			GeditViewFrame   *frame)
+{
+	GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	frame->priv->search_at_word_boundaries = gtk_check_menu_item_get_active (menu_item);
+
+	gtk_source_buffer_set_search_at_word_boundaries (buffer, frame->priv->search_at_word_boundaries);
+}
+
+static void
+match_case_toggled_cb (GtkCheckMenuItem *menu_item,
+		       GeditViewFrame   *frame)
+{
+	GtkSourceBuffer *buffer = GTK_SOURCE_BUFFER (gedit_view_frame_get_document (frame));
+
+	frame->priv->case_sensitive_search = gtk_check_menu_item_get_active (menu_item);
+
+	gtk_source_buffer_set_case_sensitive_search (buffer, frame->priv->case_sensitive_search);
 }
 
 static void
@@ -529,32 +677,47 @@ add_popup_menu_items (GtkWidget      *menu,
 
 	/* create "Wrap Around" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Wrap Around"));
-	g_signal_connect (G_OBJECT (menu_item), "toggled",
-			  G_CALLBACK (wrap_around_menu_item_toggled),
-			  frame);
+
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (wrap_around_toggled_cb),
+			  frame);
+
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
-					frame->priv->wrap_around);
+					frame->priv->search_wrap_around);
+
 	gtk_widget_show (menu_item);
 
 	/* create "Match Entire Word Only" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("Match _Entire Word Only"));
-	g_signal_connect (G_OBJECT (menu_item), "toggled",
-			  G_CALLBACK (match_entire_word_menu_item_toggled),
-			  frame);
+
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (entire_word_toggled_cb),
+			  frame);
+
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
-					GEDIT_SEARCH_IS_ENTIRE_WORD (frame->priv->search_flags));
+					frame->priv->search_at_word_boundaries);
+
 	gtk_widget_show (menu_item);
 
 	/* create "Match Case" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Match Case"));
-	g_signal_connect (G_OBJECT (menu_item), "toggled",
-			  G_CALLBACK (match_case_menu_item_toggled),
-			  frame);
+
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (match_case_toggled_cb),
+			  frame);
+
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item),
-					GEDIT_SEARCH_IS_CASE_SENSITIVE (frame->priv->search_flags));
+					frame->priv->case_sensitive_search);
+
 	gtk_widget_show (menu_item);
 }
 
@@ -658,95 +821,60 @@ search_entry_insert_text (GtkEditable    *editable,
                           gint           *position,
                           GeditViewFrame *frame)
 {
-	if (frame->priv->search_mode == GOTO_LINE)
+	gunichar c;
+	const gchar *p;
+	const gchar *end;
+	const gchar *next;
+
+	if (frame->priv->search_mode == SEARCH)
 	{
-		gunichar c;
-		const gchar *p;
-		const gchar *end;
-		const gchar *next;
+		return;
+	}
 
-		p = text;
-		end = text + length;
+	p = text;
+	end = text + length;
 
-		if (p == end)
+	if (p == end)
+	{
+		return;
+	}
+
+	c = g_utf8_get_char (p);
+
+	if (((c == '-' || c == '+') && *position == 0) ||
+	    (c == ':' && *position != 0))
+	{
+		gchar *s = NULL;
+
+		if (c == ':')
 		{
-			return;
+			s = gtk_editable_get_chars (editable, 0, -1);
+			s = g_utf8_strchr (s, -1, ':');
 		}
 
+		if (s == NULL || s == p)
+		{
+			next = g_utf8_next_char (p);
+			p = next;
+		}
+
+		g_free (s);
+	}
+
+	while (p != end)
+	{
+		next = g_utf8_next_char (p);
+
 		c = g_utf8_get_char (p);
 
-		if (((c == '-' || c == '+') && *position == 0) ||
-		    (c == ':' && *position != 0))
+		if (!g_unichar_isdigit (c))
 		{
-			gchar *s = NULL;
-
-			if (c == ':')
-			{
-				s = gtk_editable_get_chars (editable, 0, -1);
-				s = g_utf8_strchr (s, -1, ':');
-			}
-
-			if (s == NULL || s == p)
-			{
-				next = g_utf8_next_char (p);
-				p = next;
-			}
-
-			g_free (s);
+			g_signal_stop_emission_by_name (editable, "insert_text");
+			gtk_widget_error_bell (GTK_WIDGET (frame->priv->search_entry));
+			break;
 		}
 
-		while (p != end)
-		{
-			next = g_utf8_next_char (p);
-
-			c = g_utf8_get_char (p);
-
-			if (!g_unichar_isdigit (c))
-			{
-				g_signal_stop_emission_by_name (editable, "insert_text");
-				gtk_widget_error_bell (GTK_WIDGET (frame->priv->search_entry));
-				break;
-			}
-
-			p = next;
-		}
-	}
-	else
-	{
-		/* SEARCH mode */
-		static gboolean  insert_text = FALSE;
-		gchar           *escaped_text;
-		gint             new_len;
-
-		gedit_debug_message (DEBUG_SEARCH, "Text: %s", text);
-
-		/* To avoid recursive behavior */
-		if (insert_text)
-		{
-			return;
-		}
-
-		escaped_text = gedit_utils_escape_search_text (text);
-
-		gedit_debug_message (DEBUG_SEARCH, "Escaped Text: %s", escaped_text);
-
-		new_len = strlen (escaped_text);
-
-		if (new_len == length)
-		{
-			g_free (escaped_text);
-			return;
-		}
-
-		insert_text = TRUE;
-
-		g_signal_stop_emission_by_name (editable, "insert_text");
-
-		gtk_editable_insert_text (editable, escaped_text, new_len, position);
-
-		insert_text = FALSE;
-
-		g_free (escaped_text);
+		p = next;
 	}
 }
 
@@ -776,7 +904,7 @@ customize_for_search_mode (GeditViewFrame *frame)
 		gtk_widget_hide (frame->priv->go_down_button);
 	}
 
-	gtk_entry_set_icon_from_gicon (frame->priv->search_entry,
+	gtk_entry_set_icon_from_gicon (GTK_ENTRY (frame->priv->search_entry),
 	                               GTK_ENTRY_ICON_PRIMARY,
 	                               icon);
 
@@ -785,6 +913,7 @@ customize_for_search_mode (GeditViewFrame *frame)
 
 static void
 search_init (GtkWidget      *entry,
+	     GParamSpec     *pspec,
              GeditViewFrame *frame)
 {
 	const gchar *entry_text;
@@ -804,12 +933,7 @@ search_init (GtkWidget      *entry,
 	if (frame->priv->search_mode == SEARCH)
 	{
 		update_search (frame);
-
-		run_search (frame,
-		            entry_text,
-		            FALSE,
-		            frame->priv->wrap_around,
-		            TRUE);
+		start_search (frame);
 	}
 	else if (*entry_text != '\0')
 	{
@@ -909,9 +1033,41 @@ on_go_down_button_clicked (GtkWidget      *button,
 	search_again (frame, FALSE);
 }
 
+static void
+mark_set_cb (GtkTextBuffer  *buffer,
+	     GtkTextIter    *location,
+	     GtkTextMark    *mark,
+	     GeditViewFrame *frame)
+{
+	GtkTextMark *insert;
+	GtkTextMark *selection_bound;
+
+	insert = gtk_text_buffer_get_insert (buffer);
+	selection_bound = gtk_text_buffer_get_selection_bound (buffer);
+
+	if (mark == insert || mark == selection_bound)
+	{
+		install_update_entry_tag_idle (frame);
+	}
+}
+
 static void
 setup_search_widget (GeditViewFrame *frame)
 {
+	GtkTextBuffer *buffer;
+
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (frame->priv->view));
+
+	g_signal_connect (buffer,
+			  "mark-set",
+			  G_CALLBACK (mark_set_cb),
+			  frame);
+
+	g_signal_connect_swapped (buffer,
+				  "notify::search-occurrences-count",
+				  G_CALLBACK (install_update_entry_tag_idle),
+				  frame);
+
 	g_signal_connect (frame->priv->slider,
 			  "key-press-event",
 	                  G_CALLBACK (search_widget_key_press_event),
@@ -942,9 +1098,13 @@ setup_search_widget (GeditViewFrame *frame)
 	                  G_CALLBACK (search_entry_insert_text),
 	                  frame);
 
+	/* Use the "notify::text" signal, because the "changed" signal is
+	 * delayed with a GtkSearchEntry.
+	 * See https://bugzilla.gnome.org/show_bug.cgi?id=700229
+	 */
 	frame->priv->search_entry_changed_id =
 		g_signal_connect (frame->priv->search_entry,
-				  "changed",
+				  "notify::text",
 		                  G_CALLBACK (search_init),
 		                  frame);
 
@@ -1020,7 +1180,7 @@ init_search_entry (GeditViewFrame *frame)
 
 		line_str = g_strdup_printf ("%d", line + 1);
 
-		gtk_entry_set_text (frame->priv->search_entry, line_str);
+		gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry), line_str);
 
 		gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
 		                            0, -1);
@@ -1030,39 +1190,49 @@ init_search_entry (GeditViewFrame *frame)
 	else
 	{
 		/* SEARCH mode */
-		gboolean  selection_exists;
-		gchar    *find_text = NULL;
-		gchar    *old_find_text;
-		GeditSearchFlags old_find_flags = 0;
-		gint      sel_len = 0;
+		gboolean selection_exists;
+		gchar *search_text = NULL;
+		const gchar *old_search_text;
+		gint selection_len = 0;
 
-		old_find_text = gedit_document_get_search_text (GEDIT_DOCUMENT (buffer),
-		                                                &old_find_flags);
+		old_search_text = gtk_source_buffer_get_search_text (GTK_SOURCE_BUFFER (buffer));
 
-		if (old_find_flags != 0)
-		{
-			frame->priv->old_search_flags = old_find_flags;
-		}
+		frame->priv->old_case_sensitive_search =
+			gtk_source_buffer_get_case_sensitive_search (GTK_SOURCE_BUFFER (buffer));
+
+		frame->priv->old_search_at_word_boundaries =
+			gtk_source_buffer_get_search_at_word_boundaries (GTK_SOURCE_BUFFER (buffer));
+
+		frame->priv->old_search_wrap_around =
+			gtk_source_buffer_get_search_wrap_around (GTK_SOURCE_BUFFER (buffer));
 
 		selection_exists = get_selected_text (buffer,
-		                                      &find_text,
-		                                      &sel_len);
+		                                      &search_text,
+		                                      &selection_len);
 
-		if (selection_exists && (find_text != NULL) && (sel_len <= 160))
+		if (selection_exists && (search_text != NULL) && (selection_len <= 160))
 		{
-			gtk_entry_set_text (frame->priv->search_entry, find_text);
+			gchar *search_text_escaped = gtk_source_utils_escape_search_text (search_text);
+
+			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
+					    search_text_escaped);
 
 			gtk_editable_set_position (GTK_EDITABLE (frame->priv->search_entry),
 			                           -1);
+
+			g_free (search_text_escaped);
 		}
-		else if (old_find_text != NULL)
+		else if (old_search_text != NULL)
 		{
+			gchar *old_search_text_escaped = gtk_source_utils_escape_search_text (old_search_text);
+
 			g_free (frame->priv->old_search_text);
-			frame->priv->old_search_text = old_find_text;
+			frame->priv->old_search_text = old_search_text_escaped;
 			g_signal_handler_block (frame->priv->search_entry,
 			                        frame->priv->search_entry_changed_id);
 
-			gtk_entry_set_text (frame->priv->search_entry, old_find_text);
+			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
+					    old_search_text_escaped);
 
 			gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
 			                            0, -1);
@@ -1071,7 +1241,7 @@ init_search_entry (GeditViewFrame *frame)
 			                          frame->priv->search_entry_changed_id);
 		}
 
-		g_free (find_text);
+		g_free (search_text);
 	}
 }
 
@@ -1122,7 +1292,7 @@ start_interactive_search_real (GeditViewFrame *frame)
 	g_signal_handler_block (frame->priv->search_entry,
 	                        frame->priv->search_entry_changed_id);
 
-	gtk_entry_set_text (frame->priv->search_entry, "");
+	gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry), "");
 
 	g_signal_handler_unblock (frame->priv->search_entry,
 	                          frame->priv->search_entry_changed_id);
@@ -1142,6 +1312,8 @@ start_interactive_search_real (GeditViewFrame *frame)
 		g_timeout_add_seconds (GEDIT_VIEW_FRAME_SEARCH_TIMEOUT,
 				       (GSourceFunc) search_entry_flush_timeout,
 				       frame);
+
+	install_update_entry_tag_idle (frame);
 }
 
 static void
@@ -1173,13 +1345,11 @@ gedit_view_frame_class_init (GeditViewFrameClass *klass)
 	/* Bind class to template */
 	gtk_widget_class_set_template_from_resource (widget_class,
 	                                             "/org/gnome/gedit/ui/gedit-view-frame.ui");
-	gtk_widget_class_bind_child (widget_class, GeditViewFramePrivate, view);
-	gtk_widget_class_bind_child (widget_class, GeditViewFramePrivate, slider);
-	gtk_widget_class_bind_child (widget_class, GeditViewFramePrivate, search_entry);
-	gtk_widget_class_bind_child (widget_class, GeditViewFramePrivate, go_up_button);
-	gtk_widget_class_bind_child (widget_class, GeditViewFramePrivate, go_down_button);
-
-	g_type_class_add_private (object_class, sizeof (GeditViewFramePrivate));
+	gtk_widget_class_bind_template_child_private (widget_class, GeditViewFrame, view);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditViewFrame, slider);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditViewFrame, search_entry);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditViewFrame, go_up_button);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditViewFrame, go_down_button);
 }
 
 static GMountOperation *
@@ -1198,10 +1368,12 @@ gedit_view_frame_init (GeditViewFrame *frame)
 	GeditDocument *doc;
 	GdkRGBA transparent = {0, 0, 0, 0};
 
-	frame->priv = GEDIT_VIEW_FRAME_GET_PRIVATE (frame);
+	frame->priv = gedit_view_frame_get_instance_private (frame);
 
 	frame->priv->typeselect_flush_timeout = 0;
-	frame->priv->wrap_around = TRUE;
+	frame->priv->case_sensitive_search = FALSE;
+	frame->priv->search_at_word_boundaries = FALSE;
+	frame->priv->search_wrap_around = TRUE;
 
 	gtk_widget_init_template (GTK_WIDGET (frame));
 
@@ -1212,6 +1384,13 @@ gedit_view_frame_init (GeditViewFrame *frame)
 						     view_frame_mount_operation_factory,
 						     frame);
 
+	frame->priv->entry_tag = gd_tagged_entry_tag_new ("");
+
+	gd_tagged_entry_tag_set_style (frame->priv->entry_tag,
+				       "gedit-search-entry-occurrences-tag");
+
+	gd_tagged_entry_tag_set_has_close_button (frame->priv->entry_tag, FALSE);
+
 	/* Slider margin */
 	setup_search_widget (frame);
 
@@ -1278,10 +1457,13 @@ gedit_view_frame_clear_search (GeditViewFrame *frame)
 
 	doc = gedit_view_frame_get_document (frame);
 
-	gedit_document_set_search_text (doc, "", GEDIT_SEARCH_DONT_SET_FLAGS);
+	gtk_source_buffer_set_search_text (GTK_SOURCE_BUFFER (doc), "");
+
 	g_signal_handler_block (frame->priv->search_entry,
 	                        frame->priv->search_entry_changed_id);
-	gtk_entry_set_text (frame->priv->search_entry, "");
+
+	gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry), "");
+
 	g_signal_handler_unblock (frame->priv->search_entry,
 	                          frame->priv->search_entry_changed_id);
 
